#include <dbsim.h>
#include <node.h>
#include <object.h>
#include <language.h>
#include <db.h>

extern	Node* 	Nodes[MAX_NODES];
extern  File*   Relations[MAX_FILES];
int     createRel(int tupCount, int tupSize, int nodeCnt, int *nodeIds, 
		  int diskId);

///////////////////////////////////////////////////////////////////////////////
//
// form 1:  disk randomly chosen, starting node for consecutive string of 
//          nodeCnt nodes is also randomly chosen.

int  createRelation(int tupCount, int tupSize, int nodeCnt) {
  return(createRel(tupCount, tupSize, nodeCnt, NULL, -1));
}

///////////////////////////////////////////////////////////////////////////////
//
// form 2:  user specifies disk, starting node for consecutive string of 
//          nodeCnt nodes is randomly chosen.

int  createRelation(int tupCount, int tupSize, int nodeCnt, int diskId) {
  return(createRel(tupCount, tupSize, nodeCnt, NULL, diskId));
}

///////////////////////////////////////////////////////////////////////////////
//
// form 3:  user specifies disk, and specific nodes (same disk on each node)

int createRelation(int tupCount, int tupSize, int nodeCnt, 
		   int *nodeIds, int diskId) {
  return(createRel(tupCount, tupSize, nodeCnt, nodeIds, diskId));
}

///////////////////////////////////////////////////////////////////////////////
//
// The real meat:  does the work for both long and short forms

int createRel(int tupCount, int tupSize, int nodeCnt, int *nodeIds, int diskId)
{
    Node*	relNodes[MAX_NODES]; 
    int 	i, disks[MAX_DISKSPERNODE], fileId, diskCount, start;

    // Picks the disk

    diskCount = 1;
    if (diskId < 0)                  // user did not specify which disk
      disks[0] = random(0, DisksPerNode-1);
    else
      disks[0] = diskId;
      
    // Locates a free file ID slot

    for (fileId = MAX_FILES - 1; fileId >= 0; fileId--) 
      if (notafile(fileId))
	break;
    
    if (fileId < 0)
    {
	printf("aargh MAX_FILES exceeded in createRelation()\n");
	return(NOTOK);
    }

    // Picks the nodes

    if (nodeIds == NULL) {          // user did not specify which nodes
      if (nodeCnt == NumDiskNodes)  // put the relation on all the nodes
	for (i=0; i<NumDiskNodes; i++) relNodes[i] = Nodes[i];
      else {                        // pick a random starting node
	start = random(0, NumDiskNodes-nodeCnt);
        for (i=0; i<nodeCnt; i++) relNodes[i] = Nodes[i+start];
      }
    }
    else                            // user specified specific nodes
      for (i=0; i<nodeCnt; i++) relNodes[i] = Nodes[nodeIds[i]];

    // Creates the actual file and store in a well known location

    Relations[fileId] = new File (fileId, nodeCnt, &relNodes[0],
		diskCount, &disks[0], tupCount, tupSize);

    // Creates the corresponding random file with distribution = const_file, 
    // and stores it in a well known location too.

    ConstFiles[fileId] = new RandomFile;
    ConstFiles[fileId]->type = data_file;
    ConstFiles[fileId]->dist = const_file;
    ConstFiles[fileId]->fileNo = fileId;
    ConstFiles[fileId]->file = Relations[fileId]; // link it to real file
    
    return(fileId);
}


