#include "dbsim.h"
#include "node.h"
#include "mailbox.h"
#include "bufmgr.h"
#include "disk.h"

///////////////////////////////////////////////////////////////////////////////
//
//  Name:        node::node
//
//  Title:       Node class constructor
//
//  Function:    Instantiates node-owned objects and starts node processes.
//               Note that arrays of subobjects whose constructors take 
//               parameters must be initialized by "new", one at a time, so 
//               the constructor parameters can be passed (C++ restriction 
//               on arrays of objects).
//                 
//  Change Activity:
//
//     08/25/91 (KPB) - Initial version.
//     11/26/91 (DDW) - modified disks field to be an array of pointers
//			instead of an array of disks.  
//			Constructor and destructor modified accordingly.
//
///////////////////////////////////////////////////////////////////////////////

extern void nodeMgr(Node*);
extern bool PlotMode;

Node::Node(int nodenum): cpu("nodeCpu", (int) CPUsPerNode)
{
    int i;

    id = nodenum;      // remember node ID in state data
    nodeMgrPort = new Mailbox("nodeMgrMbox", this);  // get mailbox for nodemgr
    lddPort = new Mailbox("lldMbox", this);// get mailbox for deadlock detector
    bufMgr = new BufMgr(BufSize, this);
    for (i=0;i<DisksPerNode;i++) disks[i] = new Disk;

    cpu.set_servicefunc(prc_shr); // set processor sharing sched discipline

    // Starts the appropriate processes on this node
    nodeMgr(this);    // node manager process

};

Node::~Node()
{
    int i;
    for (i=0;i<DisksPerNode;i++) delete disks[i];
};

void Node::printCPUStats() 
{
  printf("  %4d  %5.3f  %8.4f  %8.4f  %8.4f  %8.4f  %8d\n",
	 id,
	 cpu.util(),
	 cpu.tput(),
	 cpu.resp(),
	 cpu.qlen(),
	 cpu.serv(),
	 cpu.completions());
}

void Node::printDiskStats() 
{
  int i;
  char plotID[256];

  for (i=0; i<DisksPerNode; i++) {
    if (PlotMode)
      sprintf(plotID, "DiskN%dD%d", id, i); // label line for graphgen tool
    else
      plotID[0] = '\0';                     // no labels for human readers
    
    printf("%s  %4d %4d ", plotID, id, i);
    disks[i]->printStats();
  }
  if (DisksPerNode > 1) printf("\n");
}
