#include "dbsim.h"
#include "object.h"
#include "node.h"
#include "split.h"
#include "language.h"
#include "db.h"
#include "op.h"
#include "query.h"

Op::~Op() 
{
    // Accumulates stats for this op into the owning query

    query->cnt_bufRef += cnt_bufRef;
    query->cnt_bufHit += cnt_bufHit;
    query->cnt_bufDirty += cnt_bufDirty;
    query->cnt_diskRead += cnt_diskRead;
    query->cnt_diskCacheHit += cnt_diskCacheHit;
    query->cnt_CPUInstr += cnt_CPUInstr;
}

Op::Op(Node* n)
{
    node = n;     // save the node
    transId = -1;
    initiator = NULL;
    outTupSize = 0;
    inTupSize = 0;
    cnt_CPUInstr = cnt_bufRef = cnt_bufHit = cnt_bufDirty = cnt_diskRead = cnt_diskCacheHit =0;
};

SelectOp::SelectOp(Node* n): Op(n)
{
    file = NULL;
    index = NULL;
    selFactor = 0.0;
    indexSelectivity = 0.0;
};

SelectOp::~SelectOp() { 
    delete splitTable;
};


ScanOp::ScanOp(Node* n): Op(n)
{
    file = NULL;
    index = NULL;
    indexSelectivity = 0.0;
};

ScanOp::~ScanOp() { 
    delete splitTable;
};

StoreOp::StoreOp(Node* n): Op(n)
{
    file = NULL;
    diskId = -1;
    sectFree = -1;
    inStreams = 0;
};

JoinOp::JoinOp(Node* n): Op(n)
{
    selFactor = 0.0;
    inTupSizeR = 0;
    inTupSizeL = 0;
    tupsPerNetPageInL = 0;
    tupsPerNetPageInR = 0;
    tupsPerNetPageOut = 0;
    curNetTupFlushed = 1;
    inStreamsL = 0;
    inStreamsR = 0;
    eofCnt=0;
};

JoinOp::~JoinOp() { };



