#include "dbsim.h"
#include "object.h"
#include "mailbox.h"
#include "msg.h"
#include "node.h"
#include "bufmgr.h"
#include "split.h"
#include "language.h"
#include "db.h"
#include "op.h"
#include "object.h"
#include "query.h"
#include "cost.h"

extern bool ResetBufferFlag;

void storeOperator(StoreOp* op)
{
    int 	pageId;
    Node* 	n;
    int 	e;	
    DataMsg*	dMsg;
    CntlMsg*	cMsg;
    int		instCnt;
    int		inTupCnt;
    int		tupleCnt;  // number of tuples processed by operator
    TIME	startTime; // time operator starts running
    TIME	writeTime; // time first page was written to disk
    bool	firstWrite;
    int		curDiskTupOut;
    int		eofCnt;	  // number of end of stream indicators encountered	
    int		tupsPerNetPageIn;
    Msg*	msg;
    int		cntRecvd;
    Mailbox*	inPort;		// mailbox for process to receive messages
    int		diskId;		// on which disk the result relation should go
    int		fileTupsPage;	// number of result tuples per page


    create("store");
    n = op->node;
    // construct mail box for process to receive messages
    inPort = new Mailbox("opMbox", n, op);

    // initializations 
    tupleCnt = 0;
    cntRecvd = 0;
    startTime = clock;
    writeTime = 9999999999.9;
    firstWrite = TRUE;
    curDiskTupOut = 0;
    eofCnt = 0;
    tupsPerNetPageIn = NETPKTSIZE/(op->inTupSize + 15);
	    // 15 for padding and alignment ???

    // extract which disk to use and how many tuples there
    // are on each page of the file
    diskId = op->diskId;
    fileTupsPage = op->file->tuplesPerPage;

    if (Flags[TRACE_OP])
      printf("store (%d) started on node %d, fileNum=%d\n",
	     identity(), n->id, op->file->fileNo);

    // begin by sending the operators's private port to the query 
    // scheduler that started the operator
    cMsg = CntlMsg::alloc();
    cMsg->opCode = NEWTASK;
    cMsg->intData = n->id; 
    cMsg->send (op->initiator, inPort);

    // consume some CPU starting the operator
    n->cpu.use(((float) INITIATE_STORE) / (Mips * 1000)); // milliseconds
    op->cnt_CPUInstr += INITIATE_STORE;

    while (eofCnt < op->inStreams)
    {
	// continue reading pages from the network until we
	// have encounterd op->inStreams end of stream markers

	// block waiting for the next message
	msg = inPort->recv(); 
	if (msg->type == CONTROL)
	{
	    // message was a control message
	    cMsg = (CntlMsg*) msg;
              printf("join/probe recvd illegal control message opcode=%d\n",
		cMsg->opCode);
	    (void) CntlMsg::free(cMsg); 
	}
	else
	{
	    // message is another packet of tuples
	    dMsg = (DataMsg*) msg;
	    cntRecvd++;

//            if (Flags[TRACE_OP])
//              printf("store (%d) on node %d received netpage (%d tuples) from node %d\n",
//		identity(), n->id, dMsg->numTuples, 
//                dMsg->sourceBox->owningNode->id);

	    // calculate amount of CPU time consumed copying the tuples
	    // from the data page to a disk pages.  This calculation
	    // is meant to mimic Gamma's time for tread(), twrite(), and bcopy()

	    inTupCnt = dMsg->numTuples;
	    instCnt = (inTupCnt * INST_TREAD)*2  // 2 for tread() & twrite()
	        + (inTupCnt * INST_BCOPY * dMsg->tupleSize); // bcopy()
            n->cpu.use(((float) instCnt) / (Mips * 1000)); // milliseconds
            op->cnt_CPUInstr += instCnt;

	    tupleCnt += inTupCnt;  // total tuples processed
	    curDiskTupOut += inTupCnt; 

	    if (curDiskTupOut >= fileTupsPage)
	    {
	    	// The output page is full.  Must do an I/O operation
	    	// capture time first data page is written to disk 
	    	if (firstWrite) {
	        	writeTime = clock;
	        	firstWrite = FALSE;
	    	}
/*
		printf("store (%d) node%d,pageId%d,diskId%d\n",
		identity(),n->id,pageId,diskId);
*/
		// write page to disk
		// first must allocate a disk page
		pageId = op->file->addPage(op->node, diskId, fileTupsPage);

		if (pageId == BAD_PAGE)
	    	      printf("Store, addPage returned error %d\n", pageId);

		// Allocate a page in the bufferpool
		e = n->bufMgr->AllocPage(diskId, pageId, op->file, LOVE);
	        if (e != OK)
		    printf("Store, allocPage returned error %d\n", e);

		// Then write into the allocated page
		e = n->bufMgr->WritePage(diskId, pageId, op);
		if (e != OK)
		        printf("Store, WritePage returned error %d\n", e);

		// Finally, unfix the page
		e = n->bufMgr->UnfixPage(diskId, pageId);
		if (e != OK)
		        printf("Store, UnfixPage returned error %d\n", e);

	    	curDiskTupOut -= fileTupsPage;  // adjust for what was written
						// to disk
	    }
	    if (dMsg->eofFlag == TRUE)
	    {
	      eofCnt++;
	      if (Flags[TRACE_OP])
		printf("Store (%d) on node %d - eofCnt = %d\n", 
		       identity(), n->id, eofCnt);
	    }
	    (void) DataMsg::free(dMsg); 
	}
    }

    // at this point we have encountered end-of-stream markers
    // from all the processes sending us tuples.  If there is
    // a partially filled page flush it to disk

    if (curDiskTupOut > 0)
    {
    	// The output page is full.  Must do an I/O operation
        // capture time first data page is written to disk 
	if (firstWrite) {
	       	writeTime = clock;
	       	firstWrite = FALSE;
	}


	// a partially filled output page remains
        // first must allocate a disk page
        pageId = op->file->addPage(op->node, diskId, curDiskTupOut);

//        if (Flags[TRACE_OP])
//	  printf("store (%d) partial page, pageid = %d, diskid = %d tuples = %d\n",
//		 identity(), pageId, diskId, curDiskTupOut);

        if (pageId == BAD_PAGE)
  	     printf("Store, addPage returned error %d\n", pageId);

        // Allocate a page in the bufferpool.
        e = n->bufMgr->AllocPage(diskId, pageId, op->file, LOVE);
        if (e != OK) printf("Store, allocPage.3 returned error %d\n",e);

	// Then write into the allocated page
        e = n->bufMgr->WritePage(diskId, pageId, op);
        if (e != OK) printf("Store, WritePage.3 returned error %d\n",e);

        // Unfix the page
        e = n->bufMgr->UnfixPage(diskId, pageId);
        if (e != OK) printf("Store, UnfixPage.3 returned error %d\n",e);
    }

    // all finished.  use some cpu time to terminate the operator cleanly
    n->cpu.use(((float) TERMINATE_STORE) / (Mips * 1000)); // milliseconds
    op->cnt_CPUInstr += TERMINATE_STORE;

    if (Flags[TRACE_OP])
      printf("store (%d) on node %d finishing, pages recvd=%d\n", 
	     identity(), n->id,cntRecvd);

    if (((StorePlan*) op->query)->storeType == REMOTE) {// this is a real store
        // Complete by flushing the result file to disk
        e = n->bufMgr->FlushFile(op->file);
        if (e != OK) printf("Store, bufMgr->FlushFile returned error %d\n",e);

        // if reset flag set, reset the buffer pool
        if (ResetBufferFlag)
        {
	   e = n->bufMgr->ResetBuffer();
	   if (e != OK) 
	     printf("NodeMgr, bufMgr->ResetBuffer returned error %d\n", e);
	}
    }

    // now send a done message to the initiator
    cMsg = CntlMsg::alloc();
    cMsg->opCode = OPDONE;
    cMsg->intData = tupleCnt;  // number of tuples written
    cMsg->floatData = writeTime;  // time of first tuple written
    cMsg->queryOp = STORE;
    cMsg->query = op->query;
    cMsg->myNodeNum = n->id;
    cMsg->fileName = op->file;
    cMsg->send (op->initiator, inPort);

    delete inPort;  // delete the mail box
    delete op;	    // delete the store operator object
}
