#include "dbsim.h"
#include "object.h"
#include "mailbox.h"
#include "msg.h"
#include "node.h"
#include "bufmgr.h"
#include "split.h"
#include "language.h"
#include "db.h"
#include "op.h"
#include "object.h"
#include "query.h"
#include "cost.h"

// simulates a user terminal. just takes tuples off the network.
// runs on the distinguished scheduler node so that the rest of the
// systems is not affected.
void terminalOperator(TermOp* op)
{
    Node* 	n;
    DataMsg*	dMsg;
    CntlMsg*	cMsg;
    TIME	startTime; // time operator starts running
    int		eofCnt;	  // number of end of stream indicators encountered
    Msg*        msg;
    int		cntRecvd;
    Mailbox*	inPort;		// mailbox for process to receive messages

    create("terminalOp");
    n = op->node;
    // construct mail box for process to receive messages
    inPort = new Mailbox("opMbox", n, op);

    // initializations 
    cntRecvd = 0;
    startTime = clock;
    eofCnt = 0;

    if (Flags[TRACE_OP])
      printf("termop (%d) started on node %d\n", identity(), n->id);

    // begin by sending the operators's private port to the query 
    // scheduler that started the operator
    cMsg = CntlMsg::alloc();
    cMsg->opCode = NEWTASK;
    cMsg->intData = n->id; 
    cMsg->send (op->initiator, inPort);

    while (eofCnt < op->inStreams)
    {
	// continue reading pages from the network until we
	// have encounterd op->inStreams end of stream markers

	// block waiting for the next message.  Use base CSIM receive method,
        // to avoid charging any cpu time.
	inPort->receive((int *) &msg); 

	if (msg->type == CONTROL)
	{
	    // message was a control message
	    cMsg = (CntlMsg*) msg;
              printf("join/probe recvd illegal control message opcode=%d\n",
		cMsg->opCode);
	    (void) CntlMsg::free(cMsg); 
	}
	else
	{
	    // message is another packet of tuples
	    dMsg = (DataMsg*) msg;
	    cntRecvd++;

//            if (Flags[TRACE_OP])
//              printf("term(%d) on node %d rcvd page(%d tuples) from node %d\n",
//		identity(), n->id, dMsg->numTuples,
//               dMsg->sourceBox->owningNode->id);

	    if (dMsg->eofFlag == TRUE)
	    {
	      eofCnt++;
	      if (Flags[TRACE_OP])
		printf("Term (%d) on node %d - eofCnt = %d\n", 
		       identity(), n->id, eofCnt);
	    }
	    (void) DataMsg::free(dMsg); 
	}
    }

    if (Flags[TRACE_OP])
      printf("term (%d) on node %d finishing, pages recvd=%d\n", 
	     identity(), n->id,cntRecvd);

    // now send a done message to the initiator
    cMsg = CntlMsg::alloc();
    cMsg->opCode = OPDONE;
    cMsg->queryOp = TERMOP;
    cMsg->query = op->query;
    cMsg->myNodeNum = n->id;
    cMsg->send (op->initiator, inPort);

    delete inPort;  // delete the mail box
    delete op;	    // delete the operator object
}



