

///////////////////////////////////////////////////////////////////////////////
//
//  Name:        mailbox.h
//
//  Title:       UW database simulator mailbox class definitions.
//
//               The mbox class is an enhancement to the CSIM mailbox class 
//               which allows us to charge CSIM CPU facilities for message 
//               receive protocol pathlength costs.  It is a generic class and
//               is not intended for external use outside of its subclasses.
//
//               The MailBox and TermMailBox classes are futher refinements
//               which just indicates whether the owner is a terminal or a
//               node (send/receive processing is different for each case).
//
//  Change Activity:
//
//     08/25/91 (KPB) - Initial version.
//
///////////////////////////////////////////////////////////////////////////////

#define NETPKTSIZE 8192

extern class Msg;
extern class Node;
extern class Op;
extern class Terminal;

class mbox: public mailbox {        // enhanced CSIM mailbox
  public:
    int type;                       // TERMINAL or NODE
   
    mbox(char* name): (name) {};
    Msg* recv();                    // special version of receive
};

///////////////////////////////////////////////////////////////////////////////

class Mailbox: public mbox {        // enhanced mbox
  public:
    Node* owningNode;
    Op* owningOp;				    
    
    Mailbox(char* name, Node* owningNode, Op* owningOp = NULL);
    Msg* recv();
};

///////////////////////////////////////////////////////////////////////////////

class TermMailbox: public mbox {    // enhanced mbox
  public:
    Terminal* owningTerm;
    
    TermMailbox(char* name, Terminal* owningTerm);
};

