// four basic message types
#define QUERY 1
#define QUERYPLAN  2
#define CONTROL  3
#define DATAPKT  4

#define SIZE_CONTROL 40
#define SIZE_DATAPKT 40
#define SIZE_QUERY 40
#define SIZE_QUERYPLAN 40

#define NETPKTSIZE 8192

#define PROTOCOLCOST 1000
#define TUPLEOVERHEAD 2

// control message types
#define OPDONE      1
#define NEWTASK     2
#define TBLSTATUS   3
#define NEWSPLIT    4
#define NOTIFYSCHED 5 
#define HTREQ       6 
#define ABORT       7 
#define COMMIT      8
#define PAGEDONE    9

// various opcodes
#define SELECT       1
#define STORE        2
#define JOIN         3
#define SPOOL        4
#define DELAY        5

extern class QueryPlan;
extern class SplitTable;
extern class File;
extern class Index;
extern class Mailbox;

class Msg {
  private:
    static int nextSeqNum;      // class wide sequence number counter
  public:
    int type;                   // message type
    int numBytes;               // total number of bytes in msg packet
    Mailbox* destBox;           // destination mailbox
    Mailbox* sourceBox;         // return mailbox of sender
    int id;                     // message identifier (for debugging)
    float timestamp;            // time when the message was sent

    Msg();
    ~Msg(){};

    void print(char* traceString);
    void send(Mailbox* dest,  Mailbox* source);
    void cheapSend(Mailbox* dest, Mailbox *source);
};

//////////////////////////////////////////////////////////////////////////////


class QueryMsg: public Msg {
  private:
    static class QueryMsg* freeMsg;
  public:
    QueryPlan*  query;          // pointer to the query plan
    File*       file;           // sent along with the local select operator
    Index*      index;          // sent along with the local select operator
    SplitTable* splitTable;     // sent along with the local select operator
    bool	needMailBox;	// TRUE if operator needs to return mailbox
    QueryMsg    *nextMsg;	// use to link messages on the free list only

    QueryMsg();
    ~QueryMsg(){};
    static QueryMsg* alloc();      // allocate an empty message
    void static free(QueryMsg*);	    // release a message buffer
    void print(char* traceString);
    void send(Mailbox* dest,  Mailbox* source);
};

//////////////////////////////////////////////////////////////////////////////

class DataMsg: public Msg {
  private:
    static class DataMsg* freeMsg;
  public:
    int 	numTuples;
    int 	tupleSize;
    bool 	eofFlag;
    DataMsg     *nextMsg;	// use to link messages on the free list only

    DataMsg();
    ~DataMsg(){};
    static DataMsg* alloc();    // allocate an empty message
    void static free(DataMsg*);	// release a message buffer
    void print(char* traceString);
    void send(Mailbox* dest,  Mailbox* source);

};

//////////////////////////////////////////////////////////////////////////////


class CntlMsg: public Msg {
  private:
    static class CntlMsg* freeMsg;
  public:
    int          opCode;		// one of the control message types
    int          intData;
    float        floatData;
    SplitTable*  splitTable;    
    QueryPlan*   query;
    int 	 queryOp;       // SELECT, STORE, JOIN
    int		 numBuckets;    // valid iff opCode is NEWSPLIT
    int		 b1Entries;     // valid iff opCode is NEWSPLIT
    int          myNodeNum;     // used by op to send node number to scheduler
    File*        fileName;      // used by local store to send back local file name
    CntlMsg     *nextMsg;	// use to link messages on the free list only

    CntlMsg();
    ~CntlMsg(){};
    static CntlMsg* alloc();    // allocate an empty message
    void static free(CntlMsg*);	// release a message buffer
    void print(char* traceString);
    void send(Mailbox* dest,  Mailbox* source);
};

